@@ -47,14 +47,6 @@ public:
     void lookupHost(const QString &hostName, QObject *receiver, const char *member);
     QHostInfo lookupCachedHostInfoFor(const QString &hostName);
     void cacheLookup(const QHostInfo &);
-    void setCacheSize(int s)
-    {
-        dnsCache.setMaxCost(s);
-    }
-    void setTTL(int _ttl)
-    {
-        ttl = _ttl;
-    }
 private Q_SLOTS:
     void queryFinished(const QHostInfo &info, Query *sender);
 
@@ -62,9 +54,12 @@ private:
     class Result;
 
     QHash<QString, Query *> openQueries;
-    QCache<QString, QPair<QHostInfo, QTime>> dnsCache;
+    struct HostCacheInfo {
+        QHostInfo hostInfo;
+        QTime time;
+    };
+    QCache<QString, HostCacheInfo> dnsCache;
     QDateTime resolvConfMTime;
-    int ttl;
 };
 
 class HostInfoAgentPrivate::Result : public QObject
@@ -256,7 +251,7 @@ QHostInfo HostInfo::lookupHost(const QString &hostName, unsigned long timeout)
         return hostInfo;
     }
 
-    // Look up the name in the KIO/KHTML DNS cache...
+    // Look up the name in the KIO DNS cache...
     hostInfo = HostInfo::lookupCachedHostInfoFor(hostName);
     if (!hostInfo.hostName().isEmpty() && hostInfo.error() == QHostInfo::NoError) {
         return hostInfo;
@@ -266,17 +261,21 @@ QHostInfo HostInfo::lookupHost(const QString &hostName, unsigned long timeout)
     std::shared_ptr<NameLookupThreadRequest> request = std::make_shared<NameLookupThreadRequest>(hostName);
     nameLookUpThread()->semaphore()->acquire();
     nameLookUpThread()->semaphore()->release();
-    QMetaObject::invokeMethod(nameLookUpThread()->worker(), "lookupHost", Qt::QueuedConnection, Q_ARG(std::shared_ptr<KIO::NameLookupThreadRequest>, request));
+    NameLookUpThreadWorker *worker = nameLookUpThread()->worker();
+    auto lookupFunc = [worker, request]() {
+        worker->lookupHost(request);
+    };
+    QMetaObject::invokeMethod(worker, lookupFunc, Qt::QueuedConnection);
     if (request->semaphore()->tryAcquire(1, timeout)) {
         hostInfo = request->result();
         if (!hostInfo.hostName().isEmpty() && hostInfo.error() == QHostInfo::NoError) {
             HostInfo::cacheLookup(hostInfo); // cache the look up...
         }
     } else {
-        QMetaObject::invokeMethod(nameLookUpThread()->worker(),
-                                  "abortLookup",
-                                  Qt::QueuedConnection,
-                                  Q_ARG(std::shared_ptr<KIO::NameLookupThreadRequest>, request));
+        auto abortFunc = [worker, request]() {
+            worker->abortLookup(request);
+        };
+        QMetaObject::invokeMethod(worker, abortFunc, Qt::QueuedConnection);
     }
 
     // qDebug() << "Name look up succeeded for" << hostName;
@@ -293,25 +292,9 @@ void HostInfo::cacheLookup(const QHostInfo &info)
     hostInfoAgentPrivate()->cacheLookup(info);
 }
 
-void HostInfo::prefetchHost(const QString &hostName)
-{
-    hostInfoAgentPrivate()->lookupHost(hostName, nullptr, nullptr);
-}
-
-void HostInfo::setCacheSize(int s)
-{
-    hostInfoAgentPrivate()->setCacheSize(s);
-}
-
-void HostInfo::setTTL(int ttl)
-{
-    hostInfoAgentPrivate()->setTTL(ttl);
-}
-
 HostInfoAgentPrivate::HostInfoAgentPrivate(int cacheSize)
     : openQueries()
     , dnsCache(cacheSize)
-    , ttl(TTL)
 {
     qRegisterMetaType<QHostInfo>();
 }
@@ -329,12 +312,12 @@ void HostInfoAgentPrivate::lookupHost(const QString &hostName, QObject *receiver
     }
 #endif
 
-    if (QPair<QHostInfo, QTime> *info = dnsCache.object(hostName)) {
-        if (QTime::currentTime() <= info->second.addSecs(ttl)) {
+    if (HostCacheInfo *info = dnsCache.object(hostName)) {
+        if (QTime::currentTime() <= info->time.addSecs(TTL)) {
             Result result;
             if (receiver) {
                 QObject::connect(&result, SIGNAL(result(QHostInfo)), receiver, member);
-                Q_EMIT result.result(info->first);
+                Q_EMIT result.result(info->hostInfo);
             }
             return;
         }
@@ -361,9 +344,9 @@ void HostInfoAgentPrivate::lookupHost(const QString &hostName, QObject *receiver
 
 QHostInfo HostInfoAgentPrivate::lookupCachedHostInfoFor(const QString &hostName)
 {
-    QPair<QHostInfo, QTime> *info = dnsCache.object(hostName);
-    if (info && info->second.addSecs(ttl) >= QTime::currentTime()) {
-        return info->first;
+    HostCacheInfo *info = dnsCache.object(hostName);
+    if (info && info->time.addSecs(TTL) >= QTime::currentTime()) {
+        return info->hostInfo;
     }
 
     // not found in dnsCache
@@ -382,7 +365,7 @@ void HostInfoAgentPrivate::cacheLookup(const QHostInfo &info)
         return;
     }
 
-    dnsCache.insert(info.hostName(), new QPair<QHostInfo, QTime>(info, QTime::currentTime()));
+    dnsCache.insert(info.hostName(), new HostCacheInfo{info, QTime::currentTime()});
 }
 
 void HostInfoAgentPrivate::queryFinished(const QHostInfo &info, Query *sender)
@@ -390,7 +373,7 @@ void HostInfoAgentPrivate::queryFinished(const QHostInfo &info, Query *sender)
     const auto host = sender->hostName();
     openQueries.remove(host);
     if (info.error() == QHostInfo::NoError) {
-        dnsCache.insert(host, new QPair<QHostInfo, QTime>(info, QTime::currentTime()));
+        dnsCache.insert(host, new HostCacheInfo{info, QTime::currentTime()});
     }
     sender->deleteLater();
 }
