@@ -15,12 +15,10 @@ using namespace KDEPrivate;
 KUrlNavigatorPathSelectorEventFilter::KUrlNavigatorPathSelectorEventFilter(QObject *parent)
     : QObject(parent)
 {
-
 }
 
 KUrlNavigatorPathSelectorEventFilter::~KUrlNavigatorPathSelectorEventFilter()
 {
-
 }
 
 bool KUrlNavigatorPathSelectorEventFilter::eventFilter(QObject *watched, QEvent *event)
@@ -34,7 +32,7 @@ bool KUrlNavigatorPathSelectorEventFilter::eventFilter(QObject *watched, QEvent
                     if (url.isValid()) {
                         menu->close();
 
-                        emit tabRequested(url);
+                        Q_EMIT tabRequested(url);
                         return true;
                     }
                 }
