@@ -101,10 +101,21 @@ public:
         m_preview = icn;
     }
 
+    bool previewHandlesSequences()
+    {
+        return m_previewHandlesSequences;
+    }
+
+    void setPreviewHandlesSequences(bool handlesSequences)
+    {
+        m_previewHandlesSequences = handlesSequences;
+    }
+
 private:
     KFileItem m_item;
     KDirModelDirNode *const m_parent;
     QIcon m_preview;
+    bool m_previewHandlesSequences = true; // First sequence is always allowed
 };
 
 // Specialization for directory nodes
@@ -131,7 +142,11 @@ public:
     void setItem(const KFileItem &item) override
     {
         KDirModelNode::setItem(item);
-        m_fsType = FsTypeUnknown;
+        if (item.isNull() || !item.url().isValid()) {
+            m_fsType = LocalFs;
+        } else {
+            m_fsType = FsTypeUnknown;
+        }
     }
 
     // If we listed the directory, the child count is known. Otherwise it can be set via setChildCount.
@@ -182,7 +197,11 @@ private:
     int m_childCount : 31;
     bool m_populated : 1;
     // Network file system? (nfs/smb/ssh)
-    mutable enum { FsTypeUnknown, LocalFs, NetworkFs } m_fsType : 3;
+    mutable enum {
+        FsTypeUnknown,
+        LocalFs,
+        NetworkFs
+    } m_fsType : 3;
 };
 
 int KDirModelNode::rowNumber() const
@@ -251,7 +270,7 @@ public:
 
     QUrl urlForNode(KDirModelNode *node) const
     {
-        /**
+        /*
          * Queries and fragments are removed from the URL, so that the URL of
          * child items really starts with the URL of the parent.
          *
@@ -605,12 +624,25 @@ void KDirModelPrivate::_k_slotDeleteItems(const KFileItemList &items)
 
     // I assume all items are from the same directory.
     // From KDirLister's code, this should be the case, except maybe emitChanges?
-    const KFileItem item = items.first();
-    Q_ASSERT(!item.isNull());
-    QUrl url = item.url();
-    KDirModelNode *node = nodeForUrl(url); // O(depth)
+
+    // We need to find first item with existing node
+    // because the first deleted item could be a hidden file not belonging to any node.
+    auto findFirstNodeAndUrl = [this](const KFileItemList &items) -> QPair<KDirModelNode *, QUrl> {
+        for (const KFileItem &item : items) {
+            Q_ASSERT(!item.isNull());
+            const QUrl url = item.url();
+            KDirModelNode *node = nodeForUrl(url); // O(depth)
+            if (node) {
+                return {node, url};
+            } else {
+                qCWarning(category) << "No node found for item that was just removed:" << url;
+            }
+        }
+        return {nullptr, QUrl()};
+    };
+
+    auto [node, url] = findFirstNodeAndUrl(items);
     if (!node) {
-        qCWarning(category) << "No node found for item that was just removed:" << url;
         return;
     }
 
@@ -636,22 +668,19 @@ void KDirModelPrivate::_k_slotDeleteItems(const KFileItemList &items)
     const int childCount = dirNode->m_childNodes.count();
     QBitArray rowNumbers(childCount, false);
     for (const KFileItem &item : items) {
-        if (!node) { // don't lookup the first item twice
-            url = item.url();
-            node = nodeForUrl(url);
-            if (!node) {
-                qCWarning(category) << "No node found for item that was just removed:" << url;
-                continue;
-            }
-            if (!node->parent()) {
-                // The root node has been deleted, but it was not first in the list 'items'.
-                // see https://bugs.kde.org/show_bug.cgi?id=196695
-                return;
-            }
+        url = item.url();
+        node = nodeForUrl(url);
+        if (!node) {
+            qCWarning(category) << "No node found for item that was just removed:" << url;
+            continue;
+        }
+        if (!node->parent()) {
+            // The root node has been deleted, but it was not first in the list 'items'.
+            // see https://bugs.kde.org/show_bug.cgi?id=196695
+            return;
         }
         rowNumbers.setBit(node->rowNumber(), 1); // O(n)
         removeFromNodeHash(node, url);
-        node = nullptr;
     }
 
     int start = -1;
@@ -897,6 +926,11 @@ QVariant KDirModel::data(const QModelIndex &index, int role) const
                 }
             }
             break;
+        case HandleSequencesRole:
+            if (index.column() == Name) {
+                return node->previewHandlesSequences();
+            }
+            break;
         case Qt::TextAlignmentRole:
             if (index.column() == Size) {
                 // use a right alignment for L2R and R2L languages
@@ -1016,6 +1050,14 @@ bool KDirModel::setData(const QModelIndex &index, const QVariant &value, int rol
             return true;
         }
         break;
+    case HandleSequencesRole:
+        if (index.column() == Name) {
+            KDirModelNode *node = static_cast<KDirModelNode *>(index.internalPointer());
+            Q_ASSERT(node);
+            node->setPreviewHandlesSequences(value.toBool());
+            return true;
+        }
+        break;
     default:
         break;
     }
@@ -1329,8 +1371,7 @@ bool KDirModel::canFetchMore(const QModelIndex &parent) const
 
     KDirModelNode *node = static_cast<KDirModelNode *>(parent.internalPointer());
     const KFileItem &item = node->item();
-    return item.isDir() && !static_cast<KDirModelDirNode *>(node)->isOnNetwork() && !static_cast<KDirModelDirNode *>(node)->isPopulated()
-        && static_cast<KDirModelDirNode *>(node)->m_childNodes.isEmpty();
+    return item.isDir() && !static_cast<KDirModelDirNode *>(node)->isPopulated() && static_cast<KDirModelDirNode *>(node)->m_childNodes.isEmpty();
 }
 
 void KDirModel::fetchMore(const QModelIndex &parent)
